@@ -5,175 +5,17 @@
 from collections import deque
 
 from traits.api import (
-    Bool, HasStrictTraits, Instance, List, Property, Str, WeakRef,
-    cached_property, Dict, TraitType, Disallow,
+    Bool, HasStrictTraits, Instance, List, Property, Str, WeakRef, Dict, 
+    Disallow,
 )
 
 from .expressions import AbstractExpression
 from .toolkit import Toolkit
-from .trait_types import EnamlInstance, EnamlEvent
+from .trait_types import (
+    EnamlEvent, LazyProperty, UserAttribute, UserEvent, ExpressionTrait,
+)
 
 
-#------------------------------------------------------------------------------
-# Expression Trait
-#------------------------------------------------------------------------------
-class ExpressionTrait(TraitType):
-    """ A custom trait type which is used to help implement expression 
-    binding. Instances of this trait are added to an object, but swap
-    themselves out and replace the old trait the first time they are
-    accessed. This allows bound expressions to be initialized in the
-    proper order without requiring an explicit initialization graph.
-
-    """
-    def __init__(self, old_trait):
-        """ Initialize an expression trait.
-
-        Parameters
-        ----------
-        old_trait : ctrait
-            The trait object that the expression trait is temporarily
-            replacing. When a 'get' or 'set' is triggered on this 
-            trait, the old trait will be restored and then the default
-            value of the expression will be applied.
-        
-        """
-        super(ExpressionTrait, self).__init__()
-        self.old_trait = old_trait
-    
-    def swapout(self, obj, name):
-        """ Restore the old trait onto the object. This method takes
-        care to make sure that listeners are copied over properly.
-
-        """
-        # The default behavior of add_trait does *almost* the right
-        # thing when it copies over notifiers when replacing the
-        # existing trait. What it fails to do is update the owner
-        # attribute of TraitChangeNotifyWrappers which are managing
-        # a bound method notifier. This means that if said notifier
-        # ever dies, it removes itself from the incorrect owner list
-        # and it will be (erroneously) called on the next dispatch
-        # cycle. The logic here makes sure that the owner attribute
-        # of such a notifier is properly updated with its new owner.
-        obj.add_trait(name, self.old_trait)
-        notifiers = obj.trait(name)._notifiers(0)
-        if notifiers is not None:
-            for notifier in notifiers:
-                if hasattr(notifier, 'owner'):
-                    notifier.owner = notifiers
-
-    def compute_default(self, obj, name):
-        """ Returns the default value as computed by the most recently
-        bound expression. If a value cannot be provided, NotImplemented
-        is returned.
-
-        """
-        res = NotImplemented
-        expr = obj._expressions[name][0]
-        if expr is not None:
-            res = expr.eval()
-        return res
-
-    def get(self, obj, name):
-        """ Handle computing the initial value for the expression trait. 
-        This method first restores the old trait, then evaluates the 
-        expression and sets the value on the trait. It then performs
-        a getattr to return the new value of the trait. If the object
-        is not yet fully initialized, the value is set quietly.
-
-        """
-        self.swapout(obj, name)
-        val = self.compute_default(obj, name)
-        if val is not NotImplemented:
-            if not obj.initialized:
-                obj.trait_setq(**{name: val})
-            else:    
-                setattr(obj, name, val)
-        return getattr(obj, name, val)
-
-    def set(self, obj, name, val):
-        """ Handle the setting of an initial value for the expression
-        trait. This method first restores the old trait, then sets
-        the value on that trait. In this case, the expression object
-        is not needed. If the object is not yet fully initialized, the 
-        value is set quietly.
-
-        """
-        self.swapout(obj, name)
-        if not obj.initialized:
-            obj.trait_setq(**{name: val})
-        else:
-            setattr(obj, name, val)
-
-
-#------------------------------------------------------------------------------
-# User Attribute and Event
-#------------------------------------------------------------------------------
-class UninitializedAttributeError(Exception):
-    """ A custom Exception used by UserAttribute to signal the access 
-    of an uninitialized attribute.
-
-    """
-    # XXX - We can't inherit from AttributeError because the local 
-    # scope object used by expressions captures an AttributeError
-    # and converts it into in a KeyError in order to implement
-    # dynamic attribute scoping. We actually want this exception
-    # to propagate.
-    pass
-
-
-class UserAttribute(EnamlInstance):
-    """ An EnamlInstance subclass that is used to implement optional 
-    attribute typing when adding a new user attribute to an Enaml 
-    component.
-
-    """
-    def get(self, obj, name):
-        """ The trait getter method. Returns the value from the object's
-        dict, or raises an uninitialized error if the value doesn't exist.
-
-        """
-        dct = obj.__dict__
-        if name not in dct:
-            self.uninitialized_error(obj, name)
-        return dct[name]
-
-    def set(self, obj, name, value):
-        """ The trait setter method. Sets the value in the object's 
-        dict if it is valid, and emits a change notification if the
-        value has changed. The first time the value is set the change
-        notification will carry None as the old value.
-
-        """
-        value = self.validate(obj, name, value)
-        dct = obj.__dict__
-        if name not in dct:
-            old = None
-        else:
-            old = dct[name]
-        dct[name] = value
-        if old != value:
-            obj.trait_property_changed(name, old, value)
-
-    def uninitialized_error(self, obj, name):
-        """ A method which raises an UninitializedAttributeError for
-        the given object and attribute name
-
-        """
-        msg = "Cannot access the uninitialized '%s' attribute of the %s object"
-        raise UninitializedAttributeError(msg % (name, obj))
-
-
-class UserEvent(EnamlEvent):
-    """ A simple EnamlEvent subclass used to distinguish between events
-    declared by the framework, and events declared by the user.
-
-    """
-    pass
-
-
-#------------------------------------------------------------------------------
-# Enaml Base Component
-#------------------------------------------------------------------------------
 class BaseComponent(HasStrictTraits):
     """ The most base class of the Enaml component hierarchy.
 
@@ -192,12 +34,14 @@ class BaseComponent(HasStrictTraits):
     parent = WeakRef('BaseComponent')
 
     #: The list of children for this component. This is a read-only
-    #: cached property that is computed based on the static list of
+    #: lazy property that is computed based on the static list of
     #: _subcomponents and the items they return by calling their
     #: 'get_actual' method. This list should not be manipulated by
     #: user code.
-    children = Property(List(Instance('BaseComponent')), 
-                        depends_on='_subcomponents:_actual_updated')
+    children = LazyProperty(
+        List(Instance('BaseComponent')), 
+        depends_on='_subcomponents:_actual_updated',
+    )
 
     #: Whether the component has been initialized or not. This will be 
     #: set to True after all of the setup() steps defined here are 
@@ -275,16 +119,15 @@ class BaseComponent(HasStrictTraits):
         """
         return self
         
-    @cached_property
     def _get_children(self):
-        """ The cached property getter for the 'children' attribute.
+        """ The lazy property getter for the 'children' attribute.
 
         This property getter returns the flattened list of components
         returned by calling 'get_actual()' on each subcomponent.
 
         """
         return sum([c.get_actual() for c in self._subcomponents], [])
-
+    
     #--------------------------------------------------------------------------
     # Component Manipulation
     #--------------------------------------------------------------------------
@@ -701,6 +544,29 @@ class BaseComponent(HasStrictTraits):
     #--------------------------------------------------------------------------
     # Auxiliary Methods 
     #--------------------------------------------------------------------------
+    def when(self, switch):
+        """ A method which returns itself or None based on the truth of
+        the argument.
+
+        This can be useful to easily turn off the effects of a component
+        if various situations such as constraints-based layout.
+
+        Parameters
+        ----------
+        switch : bool
+            A boolean which indicates whether the instance or None 
+            should be returned.
+        
+        Returns
+        -------
+        result : self or None
+            If 'switch' is boolean True, self is returned. Otherwise,
+            None is returned.
+
+        """
+        if switch:
+            return self
+    
     def traverse(self, depth_first=False):
         """ Yields all of the nodes in the tree, from this node downward.
 
